#include <stdio.h>
#include <stdlib.h>

#include <mysql.h>
#include <NdbApi.hpp>

Ndb_cluster_connection *example_init(int *pargc, char ***pargv)
{
	char *connectstr = "localhost";
	int no_retries   = 4;
	int retry_delay  = 5; 
	int verbose      = 1;

	Ndb_cluster_connection *conn;

	// initialize MySQL and Ndb client libraries
	if (ndb_init()) {
		exit(EXIT_FAILURE);
	}

	// prepare connection to cluster
	printf("trying to connect to '%s'\n", connectstr);
	conn = new Ndb_cluster_connection(connectstr);

	// initiate connection
	if (conn->connect(no_retries, retry_delay, verbose))
	{
		fprintf(stderr, "Unable to connect to cluster.");
		exit(EXIT_FAILURE);
	}
	
	// wait for the storage nodes (ndbd's)
	if (conn->wait_until_ready(30,0) < 0)
	{
		fprintf(stderr, "Cluster was not ready within 30 secs.\n");
		exit(EXIT_FAILURE);
	}

	return conn; 
}

void example_end(Ndb_cluster_connection *conn)
{
	// tear down connection
	delete conn;

	// shut down MySQL and Ndb client libraries
	ndb_end(0);
}

void bailout(const char *text)
{
  fprintf(stderr, "%s\n", text);
  exit(3);
}

const char* boundName(NdbIndexScanOperation::BoundType b)
{
  switch (b) {
  case NdbIndexScanOperation::BoundEQ: return "EQ  ";
  case NdbIndexScanOperation::BoundLT: return "LT S";
  case NdbIndexScanOperation::BoundLE: return "LE  ";
  case NdbIndexScanOperation::BoundGT: return "GT S";
  case NdbIndexScanOperation::BoundGE: return "GE  ";
  }
}


int main(int argc, char **argv)
{
  // connect
  Ndb_cluster_connection *conn;
  conn = example_init(&argc, &argv);
  if (conn == NULL) bailout("connect failed");

  // get Ndb object
  Ndb *ndb = new Ndb(conn, "test");
  if (ndb == NULL) bailout("ndb failed");
  ndb->init();

  // create transaction
  NdbTransaction *trans = ndb->startTransaction();
  if (trans == NULL) {
	NdbError e = ndb->getNdbError(); 
	bailout(e.message);
  }

  // create scan operation on index IpAddrs::IpAddrs_key
  NdbIndexScanOperation *op(trans->getNdbIndexScanOperation("i1", "ndb1"));
  if (op == NULL) bailout("getNdbIndexScanOperation failed");
  
  if (op->readTuples(NdbIndexScanOperation::LM_Exclusive) == -1) {
	bailout("readTuples failed");
  }


  // bind i attribute to variable
  int iAttr;
  NdbRecAttr *piAttr;
  if ((piAttr = op->getValue("i", (char *)&iAttr)) == 0) {
	NdbError e = op->getNdbError(); 
	bailout(e.message);	  
  }

  // now check isNULL, expecting -1 as no execute() was performed yet
  printf("isNULL: %d (expecting -1)\n", piAttr->isNULL());
  
  example_end(conn);
  
  return EXIT_SUCCESS;
}
